from time import sleep
from machine import Pin, PWM, ADC

#setup servos and pins
servo_temp = PWM(Pin(0))
servo_moist = PWM(Pin(1))
servo_temp.freq(50)
servo_moist.freq(50)

servo_temp_min = 1100 # minimum servo position
servo_temp_max = 7400 # max servo position
servo_moist_min = 5500 # moisture servo minimum value
servo_moist_mid = 4350 # moisture servo midpoint (scales are balanced)
servo_moist_max = 2500 # moisture servo maximum value

sensor_moist_pwr = Pin(22,Pin.OUT) # set high to power the moisture sensor
sensor_moist_min = 50000 #dry soil sensor value
sensor_moist_max = 25000 #wet soil sensor value

# get the default moisture level
sensor_moist_pwr.value(1)
sleep(0.1)
sensor_moist_ideal = ADC(0).read_u16() # assume moisture is ideal when powered on
sensor_moist_pwr.value(0)

#setup other vars
conversion_factor = 3.3 / (65535) # Resolution of ADC
servo_moist_pos = servo_moist_mid # track the position of the moisture servo
servo_temp_pos = servo_temp_min #track the position of the temperature servo

def valmap(value, istart, istop, ostart, ostop):
  return ostart + (ostop - ostart) * ((value - istart) / (istop - istart))

def setMoistServo(m):
    global servo_moist_pos
    sensitivity = 50
    target_pos = servo_moist_mid
    # move the scales to show moisture level
    if (m<sensor_moist_ideal+sensitivity) and (m>sensor_moist_ideal-sensitivity):
        # ideal moisture
        target_pos = servo_moist_mid
       
    elif (m>=sensor_moist_ideal+sensitivity) and (m<sensor_moist_ideal+(sensitivity*2)):
        #slightly wet
        target_pos = int((servo_moist_max-servo_moist_mid)/2)
   
    elif (m<=sensor_moist_ideal-sensitivity) and (m>sensor_moist_ideal-(sensitivity*2)):
        #slightly dry
        target_pos = int((servo_moist_mid-servo_moist_min)/2)
    elif m>sensor_moist_ideal+(sensitivity*3):
        #very wet
        target_pos = servo_moist_max
    elif m<sensor_moist_ideal+(sensitivity*3):
        #very dry
        target_pos = servo_moist_min
       
    servo_moist.duty_u16(target_pos)
    sleep(0.1)


def setTempServo(t):
    global servo_temp_pos
    if (t<12) or (t>28):
        return
    sval = int(valmap(t,12,28,servo_temp_min,servo_temp_max))
    if servo_temp_pos < sval :
        for position in range(servo_temp_pos,sval ,50):
            servo_temp.duty_u16(position)
            sleep(0.02)
        servo_temp_pos = sval
    if servo_temp_pos > sval :
        for position in range(sval ,servo_temp_pos,-50):
            servo_temp.duty_u16(position)
            sleep(0.02)
        servo_temp_pos = sval

while True:
    # get the ambient temp
    sensor_temp = int(27-((ADC(4).read_u16() * conversion_factor)-0.706)/0.001721)
    print(sensor_temp)
    setTempServo(sensor_temp)
    print(valmap(sensor_temp,16,28,servo_temp_min,servo_temp_max))
   
    # get the moisture level
    sensor_moist_pwr.value(1)
    sleep(0.1)
    current_moist = ADC(0).read_u16()
    print(current_moist)
    sensor_moist_pwr.value(0)
    sleep(2)    
    setMoistServo(current_moist)
   
for position in range(2500,4350,50):
    servo_moist.duty_u16(position)
    sleep(0.01)
sleep(1)
for position in range(4350,5500,50):
    servo_moist.duty_u16(position)
    sleep(0.01)
sleep(1)
for position in range(5500,2500,-50):
    servo_moist.duty_u16(position)
    sleep(0.01)
sleep(1)
for position in range(1000,7500,50):
    servo_temp.duty_u16(position)
    sleep(0.01)
sleep(1)
for position in range(7500,1000,-50):
    servo_temp.duty_u16(position)
    sleep(0.01)
